from typing import Any, List, Dict, Text

_convert_table = ...  # type: List[List[Text]]

_initial_table = ...  # type: List[List[Text]]

_tone_table = ...  # type: List[List[Text]]

_except_table = ...  # type: List[List[Text]]


def to_wade_glides(pinyin: Text, **kwargs: Any) -> Text: ...
