from typing import List, Text


def _replace_tone2_style_dict_to_default(string: Text) -> Text: ...


def _remove_dup_items(lst: List[Text]) -> List[Text]: ...


def _remove_dup_and_empty(lst_list: List[List[Text]]) -> List[List[Text]]: ...
